@@ -5,7 +5,7 @@ from .stages import Stages
 from .contexts import Contexts
 from .config import Configs
 from .providers import ProviderBase, StoreProvider, ProviderManager, Providers
-from .parameters import ParameterProvider, ParameterManager, Parameters
-from .secrets import SecretProvider, SecretManager, Secrets
-from .templates import TemplateProvider, TemplateManager, Templates
+from .parameters import ParameterProvider, ParameterService, Parameters
+from .secrets import SecretProvider, SecretService, Secrets
+from .templates import TemplateProvider, TemplateService, Templates
 
