@@ -1,4 +1,4 @@
-
+import os
 import json
 import yaml
 import csv
@@ -10,7 +10,11 @@ from functools import reduce
 
 
 
-def format_data(data, query, format):
+def format_data(data, query, format, compress=True):
+    def compress_single_element_lists(data):
+        if not isinstance(data, list): return data
+        if len(data) > 1: return data
+        return compress_single_element_lists(data[0])
 
     if not data: return ''
 
@@ -20,6 +24,8 @@ def format_data(data, query, format):
     else:
         result = data
 
+    if compress: result = compress_single_element_lists(result)
+
     if not result: return ''
 
     if format == 'json':
@@ -108,9 +114,10 @@ def format_data(data, query, format):
     elif format == 'shell':
 
         def quote(value):
+            if not value: return ''
             import re
             ### no quoting numbers
-            if re.match(r"^[1-9][0-9]*$", value) or re.match(r"^[0-9]*\.[0-9]*$", value):
+            if re.match(r"^[0-9]$", value) or re.match(r"^[1-9][0-9]*$", value) or re.match(r"^[0-9]*\.[0-9]+$", value):
                 return value
             ### double quote if contains single quote
             elif re.match(r"^.*[']+.*$", value):
@@ -135,22 +142,20 @@ def format_data(data, query, format):
             ### if data is dictionary with single key whose value is a list, process the child list instead
             if len(keys) == 1:
                 childList = result[keys[0]]
-                if isinstance(childList, list) and len(childList):
-                    if len(childList[0].keys()) < 2:
-                        raise DSOException(f"Unable to format data as it is incompatible with the 'shell' format.")
-                    for item in childList:
-                        key = item[list(item.keys())[0]]
-                        value = quote(item[list(item.keys())[1]])
-                        outputStream += f"{key}={value}\n"
+                if isinstance(childList, list):
+                    if childList:
+                        if len(childList[0].keys()) < 2:
+                            raise DSOException(f"Unable to format data as it is incompatible with the 'shell' format.")
+                        for item in childList:
+                            key = item[list(item.keys())[0]]
+                            value = quote(item[list(item.keys())[1]])
+                            outputStream += f"{key}={value}\n"
                 else:
-                    outputStream = '\t'.join(map(str, list(result.values())))
+                    raise NotImplementedError()
             elif len(keys) > 1:
-                outputStream = '\t'.join(map(str, list(result.values())))
-
-
+                raise NotImplementedError()
         else:
             raise DSOException(f"Unable to format data as it is incompatible with the 'shell' format.")
-        
         return outputStream
 
     else:
@@ -170,11 +175,14 @@ def read_data(input, parent_key, keys, format):
         except:
             raise DSOException(CLI_MESSAGES['InvalidFileFormat'].format(format))
 
-        if not len(data): return []
+        if not data: return []
 
-        for key in keys:
-            if not key in data[0].keys():
-                raise DSOException(CLI_MESSAGES['MissingField'].format(key))
+        if keys == ['*']: 
+            keys = data[0].keys()
+        else:
+            for key in keys:
+                if not key in data[0].keys():
+                    raise DSOException(CLI_MESSAGES['MissingField'].format(key))
 
         for row in data:
             record = {}
@@ -192,11 +200,14 @@ def read_data(input, parent_key, keys, format):
         except:
             raise DSOException(CLI_MESSAGES['InvalidFileFormat'].format(format))
 
-        if not len(data): return []
+        if not data: return []
 
-        for key in keys:
-            if not key in data[0].keys():
-                raise DSOException(CLI_MESSAGES['MissingField'].format(key))
+        if keys == ['*']: 
+            keys = data[0].keys()
+        else:
+            for key in keys:
+                if not key in data[0].keys():
+                    raise DSOException(CLI_MESSAGES['MissingField'].format(key))
 
         for row in data:
             record = {}
@@ -205,21 +216,27 @@ def read_data(input, parent_key, keys, format):
             result.append(record)
             
     elif format == 'csv':
+        if parent_key: 
+            raise NotImplementedError()
+
         try:
             data = list(csv.reader(input))
         except:
             raise DSOException(CLI_MESSAGES['InvalidFileFormat'].format(format))
 
-        if not len(data): return []
+        if not data: return []
 
-        header = data[0]
-        if len(header) < len(keys):
-            raise DSOException(CLI_MESSAGES['InvalidFileFormat'].format(format))
+        if keys == ['*']: 
+            keys = data[0]
+        else:
+            header = data[0]
+            if len(header) < len(keys):
+                raise DSOException(CLI_MESSAGES['InvalidFileFormat'].format(format))
 
-        for i in range(0, len(keys)):
-            key = keys[i]
-            if not key == header[i]:
-                raise DSOException(CLI_MESSAGES['MissingField'].format(key))
+            for i in range(0, len(keys)):
+                key = keys[i]
+                if not key == header[i]:
+                    raise DSOException(CLI_MESSAGES['MissingField'].format(key))
 
         for row in data[1:]:
             record = {}
@@ -228,6 +245,12 @@ def read_data(input, parent_key, keys, format):
             result.append(record)
 
     elif format == 'raw':
+        if parent_key: 
+            raise NotImplementedError()
+
+        if keys == ['*']: 
+            raise NotImplementedError()
+
         data = input.readlines()
         try:
             for row in data:
@@ -239,11 +262,16 @@ def read_data(input, parent_key, keys, format):
             raise DSOException(CLI_MESSAGES['InvalidFileFormat'].format(format))
 
     elif format == 'shell':
+        if keys == ['*']: 
+            raise NotImplementedError()
+
         data = input.readlines()
         try:
             for row in data:
                 record = {}
                 for i in range(0, len(keys)):
+                    if not '=' in row:
+                        raise DSOException(CLI_MESSAGES['InvalidFileFormat'].format(format))
                     record[keys[i]] = row.split('=')[i].strip()
                 result.append(record)
         except:
@@ -259,6 +287,20 @@ def read_data(input, parent_key, keys, format):
 #         raise DSOException(f"Multiple '{param.name}' {type(param)} is not allowd.")
 
 
+def validate_provided(value, name, causes=[]):
+
+    test = not bool(value)
+    if test:
+        Logger.info(CLI_MESSAGES['TryHelp'], stress = False, force=True)
+        if causes:
+            raise DSOException(CLI_MESSAGES['ArgumentsProvidedBecause'].format(', '.join(causes), name))
+        else:
+            raise DSOException(CLI_MESSAGES['ArgumentsProvided'].format(name))
+    
+    return value
+
+
+
 ### opposite validate_not_all_provided
 def validate_all_provided(values, names, causes=[]):
 
@@ -295,7 +337,6 @@ def validate_none_provided(values, names, causes=[]):
 
     test = reduce(lambda x, y: bool(x) or bool(y), values)
     if test:
-        Logger.info(CLI_MESSAGES['TryHelp'], stress = False, force=True)
         Logger.info(CLI_MESSAGES['TryHelp'], stress = False, force=True)
         if causes:
             raise DSOException(CLI_MESSAGES['ArgumentsNoneProvidedBecause'].format(', '.join(causes), ', '.join(names)))
@@ -349,3 +390,25 @@ def validate_query_argument(query, query_all, default_query):
             raise DSOException(f"Invalid JMESPath query '{_query}': {e.msg}")
     
     return _query
+
+
+
+def convert_context_overrides(namespace, project, application):
+    result = ''
+    if namespace: 
+        result += f"namespace={namespace},"
+    elif 'DSO_NAMESPACE' in os.environ:
+        Logger.debug("Environment variable 'DSO_NAMESPACE' found.")
+        result += f"namespace={os.getenv('DSO_NAMESPACE')},"
+    if project: 
+        result += f"project={project},"
+    elif 'DSO_PROJECT' in os.environ:
+        Logger.debug("Environment variable 'DSO_PROJECT' found.")
+        result += f"project={os.getenv('DSO_PROJECT')},"
+    if application: 
+        result += f"application={application},"
+    elif 'DSO_APPLICATION' in os.environ:
+        Logger.debug("Environment variable 'DSO_APPLICATION' found.")
+        result += f"application={os.getenv('DSO_APPLICATION')},"    
+    
+    return result
